@@ -213,7 +213,12 @@ parse_addr_list (GPtrArray *array4, GPtrArray *array6, const char *str)
 static inline gboolean
 is_domain_valid (const char *str)
 {
-	return (str && (strlen(str) >= 1) && (strlen(str) <= 255));
+	gsize l;
+
+	if (!str)
+		return FALSE;
+	l = strlen (str);
+	return (l >= 1 && l <= 255);
 }
 
 static GVariant *
@@ -448,6 +453,7 @@ main (int argc, char *argv[])
 	gboolean has_ip4_address = FALSE;
 	gboolean has_ip6_address = FALSE;
 	gchar *bus_name = NM_DBUS_SERVICE_OPENVPN;
+	gsize size;
 
 #if !GLIB_CHECK_VERSION (2, 35, 0)
 	g_type_init ();
@@ -673,27 +679,29 @@ main (int argc, char *argv[])
 	dns6_list = g_ptr_array_new ();
 	nbns_list = g_ptr_array_new ();
 
-	for (i = 1; i < 256; i++) {
-		char *env_name;
+	for (i = 1; TRUE; i++) {
+		char env_name[NM_STRLEN ("foreign_option_") + 1 + 40];
+
+		nm_sprintf_buf (env_name, "foreign_option_%d", i);
 
-		env_name = g_strdup_printf ("foreign_option_%d", i);
 		tmp = getenv (env_name);
-		g_free (env_name);
 
-		if (!tmp || strlen (tmp) < 1)
+		if (   !tmp
+		    || !tmp[0])
 			break;
 
 		if (!g_str_has_prefix (tmp, "dhcp-option "))
 			continue;
 
-		tmp += 12; /* strlen ("dhcp-option ") */
+		tmp += NM_STRLEN ("dhcp-option ");
 
 		if (g_str_has_prefix (tmp, "DNS "))
-			parse_addr_list (dns4_list, dns6_list, tmp + 4);
+			parse_addr_list (dns4_list, dns6_list, &tmp[NM_STRLEN ("DNS ")]);
 		else if (g_str_has_prefix (tmp, "WINS "))
-			parse_addr_list (nbns_list, NULL, tmp + 5);
-		else if (g_str_has_prefix (tmp, "DOMAIN ") && is_domain_valid (tmp + 7))
-			g_ptr_array_add (dns_domains, tmp + 7);
+			parse_addr_list (nbns_list, NULL, &tmp[NM_STRLEN ("WINS ")]);
+		else if (   g_str_has_prefix (tmp, "DOMAIN ")
+		         && is_domain_valid (&tmp[NM_STRLEN ("DOMAIN ")]))
+			g_ptr_array_add (dns_domains, &tmp[NM_STRLEN ("DOMAIN ")]);
 	}
 
 	if (dns4_list->len) {
@@ -744,22 +752,28 @@ main (int argc, char *argv[])
 
 	ip4config = g_variant_builder_end (&ip4builder);
 
-	if (g_variant_n_children (ip4config)) {
-		val = g_variant_new_boolean (TRUE);
-		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_HAS_IP4, val);
-	} else {
+	size = g_variant_n_children (ip4config);
+	if (size == 0 || !has_ip4_address) {
+		if (size > 0)
+			_LOGW ("Ignoring IPv4 configuration without an address");
 		g_variant_unref (ip4config);
 		ip4config = NULL;
+	} else {
+		val = g_variant_new_boolean (TRUE);
+		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_HAS_IP4, val);
 	}
 
 	ip6config = g_variant_builder_end (&ip6builder);
 
-	if (g_variant_n_children (ip6config)) {
-		val = g_variant_new_boolean (TRUE);
-		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_HAS_IP6, val);
-	} else {
+	size = g_variant_n_children (ip6config);
+	if (size == 0 || !has_ip6_address) {
+		if (size > 0)
+			_LOGW ("Ignoring IPv6 configuration without an address");
 		g_variant_unref (ip6config);
 		ip6config = NULL;
+	} else {
+		val = g_variant_new_boolean (TRUE);
+		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_HAS_IP6, val);
 	}
 
 	if (!ip4config && !ip6config)
