@@ -138,6 +138,7 @@ typedef struct {
 } ValidProperty;
 
 static const ValidProperty valid_properties[] = {
+	{ NM_OPENVPN_KEY_ALLOW_PULL_FQDN,           G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_AUTH,                      G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_CA,                        G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_CERT,                      G_TYPE_STRING, 0, 0, FALSE },
@@ -167,9 +168,11 @@ static const ValidProperty valid_properties[] = {
 	{ NM_OPENVPN_KEY_PROXY_SERVER,              G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_PROXY_PORT,                G_TYPE_INT, 1, 65535, FALSE },
 	{ NM_OPENVPN_KEY_PROXY_RETRY,               G_TYPE_BOOLEAN, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_PUSH_PEER_INFO,            G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_HTTP_PROXY_USERNAME,       G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_REMOTE,                    G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_REMOTE_RANDOM,             G_TYPE_BOOLEAN, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_REMOTE_RANDOM_HOSTNAME,    G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_REMOTE_IP,                 G_TYPE_STRING, 0, 0, TRUE },
 	{ NM_OPENVPN_KEY_RENEG_SECONDS,             G_TYPE_INT, 0, G_MAXINT, FALSE },
 	{ NM_OPENVPN_KEY_STATIC_KEY,                G_TYPE_STRING, 0, 0, FALSE },
@@ -182,6 +185,7 @@ static const ValidProperty valid_properties[] = {
 	{ NM_OPENVPN_KEY_TUN_IPV6,                  G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TLS_CIPHER,                G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TLS_CRYPT,                 G_TYPE_STRING, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_TLS_CRYPT_V2,              G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TLS_REMOTE,                G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_VERIFY_X509_NAME,          G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_REMOTE_CERT_TLS,           G_TYPE_STRING, 0, 0, FALSE },
@@ -354,18 +358,14 @@ args_add_vpn_certs (GPtrArray *args, NMSettingVpn *s_vpn)
 	key  = nm_utils_str_utf8safe_unescape (key,  &key_free);
 
 	if (   nmovpn_arg_is_set (ca)
-	    && nmovpn_arg_is_set (cert)
-	    && nmovpn_arg_is_set (key)
-	    && nm_streq (ca, cert)
-	    && nm_streq (ca, key))
-		args_add_strv (args, "--pkcs12", ca);
+	    && !is_pkcs12 (ca))
+		args_add_strv (args, "--ca", ca);
+
+	if (nmovpn_arg_is_set (cert) && is_pkcs12 (cert))
+		args_add_strv (args, "--pkcs12", cert);
 	else {
-		if (nmovpn_arg_is_set (ca))
-			args_add_strv (args, "--ca", ca);
-		if (nmovpn_arg_is_set (cert))
-			args_add_strv (args, "--cert", cert);
-		if (nmovpn_arg_is_set (key))
-			args_add_strv (args, "--key", key);
+		args_add_strv (args, "--cert", cert);
+		args_add_strv (args, "--key", key);
 	}
 }
 
@@ -1437,8 +1437,10 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 				}
 			} else if (nm_streq0 (proto_tcp, "yes"))
 				args_add_strv (args, "tcp-client");
-			else
+			else {
 				args_add_strv (args, "udp");
+				args_add_strv (args, "--explicit-exit-notify");
+			}
 		}
 	}
 
@@ -1446,6 +1448,14 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	if (nm_streq0 (tmp, "yes"))
 		args_add_strv (args, "--remote-random");
 
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_REMOTE_RANDOM_HOSTNAME);
+	if (nm_streq0 (tmp, "yes"))
+		args_add_strv (args, "--remote-random-hostname");
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_ALLOW_PULL_FQDN);
+	if (nm_streq0 (tmp, "yes"))
+		args_add_strv (args, "--allow-pull-fqdn");
+
 	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TUN_IPV6);
 	if (nm_streq0 (tmp, "yes"))
 		args_add_strv (args, "--tun-ipv6");
@@ -1690,6 +1700,12 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		args_add_strv (args, "--tls-crypt");
 		args_add_utf8safe_str (args, tmp);
 	}
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_CRYPT_V2);
+	if (nmovpn_arg_is_set (tmp)) {
+		args_add_strv (args, "--tls-crypt-v2");
+		args_add_utf8safe_str (args, tmp);
+	}
 	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN);
 	if (nmovpn_arg_is_set (tmp)) {
 		args_add_strv (args, "--tls-version-min");
@@ -1879,6 +1895,10 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	args_add_strv (args, "--route-noexec");
 	args_add_strv (args, "--ifconfig-noexec");
 
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PUSH_PEER_INFO);
+	if (nm_streq0 (tmp, "yes"))
+		args_add_strv (args, "--push-peer-info");
+
 	/* Now append configuration options which are dependent on the configuration type */
 	if (nm_streq (connection_type, NM_OPENVPN_CONTYPE_TLS)) {
 		args_add_strv (args, "--client");
